@@ -48,6 +48,8 @@ static void parse_rc_file(void)
 {
 	if (strlen(Xdialog.rc_file) != 0)
 		gtk_rc_parse(Xdialog.rc_file);
+	if (dialog_compat)
+		gtk_rc_parse_string(FIXED_FONT_RC_STRING);
 }
 
 /* font_init() is used for two purposes: load the fixed font that Xdialog may
@@ -60,6 +62,7 @@ static void font_init(void)
 {
 	GtkWidget *window;
 	GtkStyle  *style;
+	GdkFont *font;
 	gint width, ascent, descent, lbearing, rbearing;
 
 	fixed_font = gdk_font_load(FIXED_FONT);
@@ -82,7 +85,12 @@ static void font_init(void)
 		style = window->style;
 		if (style != NULL) {
 			/* For proportionnal fonts, we use the average character width... */
-			gdk_string_extents(style->font, ALPHANUM_CHARS, &lbearing,
+#ifdef USE_GTK2
+			font = gtk_style_get_font(style);
+#else
+			font = style->font;
+#endif
+			gdk_string_extents(font, ALPHANUM_CHARS, &lbearing,
 					   &rbearing, &width, &ascent, &descent);
 			xmult = width / 62;		/* 62 = strlen(ALPHANUM_CHARS) */
 			ymult = ascent + descent + 2;	/*  2 = spacing pixel lines */
@@ -98,7 +106,11 @@ static void wrap_text(gchar *str, gint reserved_width)
 	gint max_line_width, n = 0;
 	gchar *p = str, *last_space = NULL;
 	gchar tmp[MAX_LABEL_LENGTH];
+#ifdef USE_GTK2
+	GdkFont *current_font = gtk_style_get_font(Xdialog.window->style);
+#else
 	GdkFont *current_font = Xdialog.window->style->font;
+#endif
 
 	if (Xdialog.xsize != 0)
 		max_line_width = (Xdialog.size_in_pixels ? Xdialog.xsize :
@@ -164,14 +176,14 @@ static void open_window(void)
 	GtkWidget *window;
 	GtkWidget *vbox;
 
-	parse_rc_file();
 	font_init();
-	if (dialog_compat)
-		gtk_rc_parse_string(FIXED_FONT_RC_STRING);
 
 	/* Open a new GTK top-level window */
 	window = Xdialog.window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 
+	/* Apply the custom GTK+ theme, if any. */
+	parse_rc_file();
+
 	if (Xdialog.wmclass[0] != 0)
 		gtk_window_set_wmclass(GTK_WINDOW(window), Xdialog.wmclass,
 				       Xdialog.wmclass);
@@ -501,6 +513,22 @@ static GtkWidget *set_all_buttons(gboolean print, gboolean ok)
 	return button_ok;
 }
 
+#ifdef USE_GTK2
+static GtkWidget *set_scrollable_text(void)
+{
+	GtkWidget *text;
+	GtkWidget *scrollwin;
+
+	scrollwin = gtk_scrolled_window_new(NULL, NULL);
+	gtk_widget_show (scrollwin);
+	gtk_box_pack_start (GTK_BOX(Xdialog.vbox), scrollwin, TRUE, TRUE, 0);
+
+	text = gtk_text_view_new();
+	gtk_widget_show(text);
+	gtk_container_add(GTK_CONTAINER (scrollwin), text);
+	return text;
+}
+#else
 static GtkWidget *set_scrollable_text(void)
 {
 	GtkWidget *hbox;
@@ -531,6 +559,7 @@ static GtkWidget *set_scrollable_text(void)
 
 	return text;
 }
+#endif
 
 static GtkWidget *set_scrolled_window(GtkBox *box, gint border_width, gint xsize,
 				      gint list_size, gint spacing)
@@ -555,6 +584,38 @@ static GtkWidget *set_scrolled_window(GtkBox *box, gint border_width, gint xsize
 	return scrolled_window;
 }
 
+#ifdef USE_GTK2
+static GtkWidget *set_scrolled_list(GtkWidget *box, gint xsize, gint list_size,
+				    gint spacing, GtkListStore *store)
+{
+	GtkWidget *scrolled_window;
+	GtkWidget *list;
+	GtkCellRenderer *renderer;
+	GtkTreeViewColumn *column;
+	GtkTreeSelection* selection;
+
+	scrolled_window = set_scrolled_window(GTK_BOX(box), 0, xsize,
+		list_size, spacing);
+	list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
+
+	renderer = gtk_cell_renderer_text_new();
+	column = gtk_tree_view_column_new_with_attributes("", renderer, "text",
+		0, NULL);
+
+	gtk_tree_view_append_column(GTK_TREE_VIEW(list), column);
+
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(list));
+	/* TODO: This seems to screw things up. At the moment it is not important
+	   to have multiple selections */
+	/* gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE); */
+
+	gtk_widget_show(list);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window),
+		list);
+
+	return list;
+}
+#else
 static GtkWidget *set_scrolled_list(GtkWidget *box, gint xsize, gint list_size,
 				    gint spacing)
 {
@@ -569,6 +630,7 @@ static GtkWidget *set_scrolled_list(GtkWidget *box, gint xsize, gint list_size,
 
 	return list;
 }
+#endif
 
 static GtkObject *set_horizontal_slider(GtkBox *box, gint deflt, gint min, gint max)
 {
@@ -646,6 +708,12 @@ static int item_status(GtkWidget *item, char *status, char *tag)
 		return 0;
 }
 
+static void set_timeout(void)
+{
+	if (Xdialog.timeout > 0)
+		Xdialog.timer2 = gtk_timeout_add(Xdialog.timeout*1000, timeout_exit, NULL);
+}
+
 /*
  * The Xdialog widgets...
  */
@@ -677,6 +745,8 @@ void create_msgbox(gchar *optarg, gboolean yesno)
 
 	if (Xdialog.help)
 		set_button(HELP, hbuttonbox, 2, FALSE);
+
+	set_timeout();
 }
 
 
@@ -862,6 +932,8 @@ void create_tailbox(gchar *optarg)
 		set_all_buttons(TRUE, Xdialog.ok_button);
 
 	Xdialog.timer = gtk_timeout_add(10, (GtkFunction) tailbox_timeout, NULL);
+
+	set_timeout();
 }
 
 
@@ -922,12 +994,19 @@ void create_logbox(gchar *optarg)
 		set_all_buttons(FALSE, Xdialog.ok_button);
 
 	Xdialog.timer = gtk_timeout_add(10, (GtkFunction) logbox_timeout, NULL);
+
+	set_timeout();
 }
 
 
 void create_textbox(gchar *optarg, gboolean editable)
 {
+#ifdef USE_GTK2
+	GtkTextView *text;
+	GtkTextBuffer *text_buffer;
+#else
 	GtkText *text;
+#endif
 	GtkWidget *button_ok = NULL;
 	FILE *infile;
 	gint i, n = 0, llen = 0, lcnt = 0;
@@ -938,10 +1017,17 @@ void create_textbox(gchar *optarg, gboolean editable)
 
 	Xdialog.widget1 = set_scrollable_text();
 	gtk_widget_grab_focus(Xdialog.widget1);
+#ifdef USE_GTK2
+	text = GTK_TEXT_VIEW(Xdialog.widget1);
+	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(text));
+#else
 	text = GTK_TEXT(Xdialog.widget1);
+#endif
 
 	/* Fill the GtkText with the text */
+#ifndef USE_GTK2
 	gtk_text_freeze(text);
+#endif
 	if (strcmp(optarg, "-") == 0)
 		infile = stdin;
 	else
@@ -952,7 +1038,11 @@ void create_textbox(gchar *optarg, gboolean editable)
 
 		do {
 			nchars = fread(buffer, 1, 1024, infile);
+#ifdef USE_GTK2
+			gtk_text_buffer_insert_at_cursor(text_buffer, buffer, nchars);
+#else
 			gtk_text_insert(text, NULL, NULL, NULL, buffer, nchars);
+#endif
 
 			/* Calculate the maximum line length and lines count */
 			for (i = 0; i < nchars; i++)
@@ -972,7 +1062,9 @@ void create_textbox(gchar *optarg, gboolean editable)
 		if (infile != stdin)
 			fclose(infile);
 	}
+#ifndef USE_GTK2
 	gtk_text_thaw(text);
+#endif
 	llen += 4;
 	if (Xdialog.fixed_font)
 		gtk_widget_set_usize(Xdialog.widget1,
@@ -984,7 +1076,11 @@ void create_textbox(gchar *optarg, gboolean editable)
 				     MIN(lcnt*ymult, gdk_screen_height()-10*ymult));
 
 	/* Set the editable flag depending on what we want (text or edit box) */
+#ifdef USE_GTK2
+	gtk_text_view_set_editable(text, editable);
+#else
 	gtk_text_set_editable(text, editable);
+#endif
 
 	if (dialog_compat && !editable)
 		Xdialog.cancel_button = FALSE;
@@ -996,6 +1092,8 @@ void create_textbox(gchar *optarg, gboolean editable)
 	if (editable)
 		gtk_signal_connect(GTK_OBJECT(button_ok), "clicked",
 				   GTK_SIGNAL_FUNC(editbox_ok), NULL);
+
+	set_timeout();
 }
 
 
@@ -1024,8 +1122,11 @@ void create_inputbox(gchar *optarg, gchar *options[], gint entries)
 
 	Xdialog.widget1 = gtk_entry_new();
 	entry = GTK_ENTRY(Xdialog.widget1);
+#ifndef USE_GTK2
+	/* Would kill the entry of text in GTK2, so we enable it for GTK+ only. */
 	gtk_signal_connect(GTK_OBJECT(Xdialog.widget1), "key_press_event",
 			   GTK_SIGNAL_FUNC(input_keypress), NULL);
+#endif
 	gtk_entry_set_text(entry, deftext);
 	gtk_box_pack_start(Xdialog.vbox, Xdialog.widget1, TRUE, TRUE, 0);
 	gtk_widget_grab_focus(Xdialog.widget1);
@@ -1036,8 +1137,10 @@ void create_inputbox(gchar *optarg, gchar *options[], gint entries)
 
 		Xdialog.widget2 = gtk_entry_new();
 		entry = GTK_ENTRY(Xdialog.widget2);
+#ifndef USE_GTK2
 		gtk_signal_connect(GTK_OBJECT(Xdialog.widget2), "key_press_event",
 				   GTK_SIGNAL_FUNC(input_keypress), NULL);
+#endif
 		gtk_entry_set_text(entry, options[3]);
 		gtk_box_pack_start(Xdialog.vbox, Xdialog.widget2, TRUE, TRUE, 0);
 		gtk_widget_show(Xdialog.widget2);
@@ -1049,8 +1152,10 @@ void create_inputbox(gchar *optarg, gchar *options[], gint entries)
 
 		Xdialog.widget3 = gtk_entry_new();
 		entry = GTK_ENTRY(Xdialog.widget3);
+#ifndef USE_GTK2
 		gtk_signal_connect(GTK_OBJECT(Xdialog.widget3), "key_press_event",
 				   GTK_SIGNAL_FUNC(input_keypress), NULL);
+#endif
 		gtk_entry_set_text(entry, options[5]);
 		gtk_box_pack_start(Xdialog.vbox, Xdialog.widget3, TRUE, TRUE, 0);
 		gtk_widget_show(Xdialog.widget3);
@@ -1073,6 +1178,8 @@ void create_inputbox(gchar *optarg, gchar *options[], gint entries)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, inputbox_timeout, NULL);
+
+	set_timeout();
 }
 
 
@@ -1111,6 +1218,8 @@ void create_combobox(gchar *optarg, gchar *options[], gint list_size)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, inputbox_timeout, NULL);
+
+	set_timeout();
 }
 
 
@@ -1169,6 +1278,8 @@ void create_rangebox(gchar *optarg, gchar *options[], gint ranges)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, rangebox_timeout, NULL);
+
+	set_timeout();
 }
 
 
@@ -1212,6 +1323,8 @@ void create_spinbox(gchar *optarg, gchar *options[], gint spins)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, spinbox_timeout, NULL);
+
+	set_timeout();
 }
 
 
@@ -1281,6 +1394,8 @@ void create_itemlist(gchar *optarg, gint type, gchar *options[], gint list_size)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, itemlist_timeout, NULL);
+
+	set_timeout();
 }
 
 
@@ -1291,9 +1406,15 @@ void create_buildlist(gchar *optarg, gchar *options[], gint list_size)
 	GtkWidget *button_add;
 	GtkWidget *button_remove;
 	GtkWidget *button_ok;
+#ifdef USE_GTK2
+	GtkListStore *tree_list1;
+	GtkListStore *tree_list2;
+	GtkTreeIter tree_iter;
+#else
 	GList *glist1 = NULL;
 	GList *glist2 = NULL;
 	GtkWidget *item;
+#endif
 	GtkTooltips *tooltips = NULL;
 	gint i, n = 0;
 	int params = 3 + Xdialog.tips;
@@ -1308,12 +1429,23 @@ void create_buildlist(gchar *optarg, gchar *options[], gint list_size)
 	set_backtitle(TRUE);
 	set_label(optarg, FALSE);
 
+#ifdef USE_GTK2
+	tree_list1 = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	tree_list2 = gtk_list_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+#endif
+
 	/* Put all parameters into an array and calculate the max item width */
 	for (i = 0;  i < list_size; i++) {
 		strcpysafe(Xdialog.array[i].tag, options[params*i], MAX_ITEM_LENGTH);
 		strcpysafe(Xdialog.array[i].name, options[params*i+1], MAX_ITEM_LENGTH);
 		if (strlen(Xdialog.array[i].name) > n)
 			n = strlen(Xdialog.array[i].name);
+#ifdef USE_GTK2
+		gtk_list_store_append(tree_list1, &tree_iter);
+		gtk_list_store_set(tree_list1, &tree_iter, 0,
+			Xdialog.array[i].name, 1, Xdialog.array[i].tag, -1);
+		/* TODO: tooltips support in GTK2 */
+#else
 		item = gtk_list_item_new_with_label(Xdialog.array[i].name);
 		gtk_widget_show(item);
 		Xdialog.array[i].widget = item;
@@ -1324,6 +1456,7 @@ void create_buildlist(gchar *optarg, gchar *options[], gint list_size)
 
 		if (Xdialog.tips == 1 && strlen(options[params*i+3]) > 0)
 			gtk_tooltips_set_tip(tooltips, item, (gchar *) options[params*i+3], NULL);
+#endif
 	}
 
 	/* Setup a hbox to hold the scrolled windows and the Add/Remove buttons */
@@ -1332,7 +1465,12 @@ void create_buildlist(gchar *optarg, gchar *options[], gint list_size)
 	gtk_box_pack_start(Xdialog.vbox, hbox, TRUE, TRUE, ymult/3);
 
 	/* Setup the first list into a scrolled window */
+#ifdef USE_GTK2
+	Xdialog.widget1 = set_scrolled_list(hbox, MAX(15, n), list_size, 4, tree_list1);
+	g_object_unref(G_OBJECT(tree_list1));
+#else
 	Xdialog.widget1 = set_scrolled_list(hbox, MAX(15, n), list_size, 4);
+#endif
 
 	/* Setup the Add/Remove buttons */
 	vbuttonbox = gtk_vbutton_box_new();
@@ -1347,18 +1485,27 @@ void create_buildlist(gchar *optarg, gchar *options[], gint list_size)
 			   GTK_SIGNAL_FUNC(remove_from_list), NULL);
 
 	/* Setup the second list into a scrolled window */
+#ifdef USE_GTK2
+	Xdialog.widget2 = set_scrolled_list(hbox, MAX(15, n), list_size, 4, tree_list2);
+	g_object_unref(G_OBJECT(tree_list2));
+#else
 	Xdialog.widget2 = set_scrolled_list(hbox, MAX(15, n), list_size, 4);
+#endif
 
 	button_ok = set_all_buttons(FALSE, TRUE);
 	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked", GTK_SIGNAL_FUNC(print_list), NULL);
 
+#ifndef USE_GTK2
 	gtk_list_append_items(GTK_LIST(Xdialog.widget1), glist1);
 	gtk_list_append_items(GTK_LIST(Xdialog.widget2), glist2);
+#endif
 
 	sensitive_buttons();
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, buildlist_timeout, NULL);
+
+	set_timeout();
 }
 
 
@@ -1487,9 +1634,95 @@ void create_menubox(gchar *optarg, gchar *options[], gint list_size)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, menu_timeout, NULL);
+
+	set_timeout();
 }
 
+#ifdef USE_GTK2
+/* TODO: implement tooltips support */
+void create_treeview(gchar *optarg, gchar *options[], gint list_size)
+{
+	GtkWidget *scrolled_window;
+	GtkWidget *button_ok;
+	GtkTreeStore *store;
+	GtkTreeViewColumn *column;
+	GtkCellRenderer *renderer;
+	GtkTreeIter tree_iter[MAX_TREE_DEPTH];
+	GtkTreeSelection *select;
+/*	GtkTooltips *tooltips = NULL; */
+	int depth = 0;
+	int i;
+	int params = 4 + Xdialog.tips;
 
+/*	if (Xdialog.tips == 1)
+		tooltips = gtk_tooltips_new(); */
+
+	Xdialog_array(list_size);
+
+	open_window();
+
+	set_backtitle(TRUE);
+	set_label(optarg, FALSE);
+
+	/* Fill the store with the data */
+	store = gtk_tree_store_new(2, G_TYPE_STRING, G_TYPE_STRING);
+	for (i = 0 ; i < list_size ; i++) {
+		strcpysafe(Xdialog.array[i].tag, options[params*i], MAX_ITEM_LENGTH);
+		strcpysafe(Xdialog.array[i].name, options[params*i+1], MAX_ITEM_LENGTH);
+		Xdialog.array[i].state = 0;
+
+		depth = atoi(options[params*i+3]);
+
+		if (depth > MAX_TREE_DEPTH) {
+			fprintf(stderr,
+				"Xdialog: Max allowed depth for "\
+				"--treeview is %d !  Aborting...\n",
+				MAX_TREE_DEPTH);
+			exit(255);
+		}
+
+		if (depth == 0) {
+			gtk_tree_store_append(store, &tree_iter[0], NULL);
+			gtk_tree_store_set(store, &tree_iter[0], 0,
+				Xdialog.array[i].name, 1, Xdialog.array[i].tag, -1);
+		} else {
+			gtk_tree_store_append(store, &tree_iter[depth],
+				&tree_iter[depth-1]);
+			gtk_tree_store_set(store, &tree_iter[depth], 0,
+				Xdialog.array[i].name, 1, Xdialog.array[i].tag, -1);
+		}
+	}     
+
+	/* Create the tree view in a scrolled window */
+	scrolled_window = set_scrolled_window(Xdialog.vbox, xmult/2, -1, list_size, 4);
+
+	Xdialog.widget1 = gtk_tree_view_new_with_model(GTK_TREE_MODEL (store));
+	g_object_unref(G_OBJECT (store));
+
+	renderer = gtk_cell_renderer_text_new();
+
+	column = gtk_tree_view_column_new_with_attributes ("", renderer, "text", 0, NULL);
+
+	gtk_tree_view_append_column(GTK_TREE_VIEW(Xdialog.widget1), column);
+
+	gtk_widget_show(Xdialog.widget1);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrolled_window), Xdialog.widget1);
+
+	/* Create and hookup the ok button */
+	button_ok = set_all_buttons(FALSE, TRUE);
+	gtk_signal_connect(GTK_OBJECT(button_ok), "clicked", GTK_SIGNAL_FUNC(print_tree_selection), NULL);
+
+	/* Setup the selection handler */
+	select = gtk_tree_view_get_selection(GTK_TREE_VIEW(Xdialog.widget1));
+	gtk_tree_selection_set_mode(select, GTK_SELECTION_SINGLE);
+	g_signal_connect(G_OBJECT(select), "changed", G_CALLBACK(cb_selection_changed), NULL);
+
+	if (Xdialog.interval > 0)
+		Xdialog.timer = gtk_timeout_add(Xdialog.interval, menu_timeout, NULL);
+
+	set_timeout();
+}
+#else
 void create_treeview(gchar *optarg, gchar *options[], gint list_size)
 {
 	GtkWidget *scrolled_window;
@@ -1605,7 +1838,10 @@ void create_treeview(gchar *optarg, gchar *options[], gint list_size)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, menu_timeout, NULL);
+
+	set_timeout();
 }
+#endif
 
 
 void create_filesel(gchar *optarg, gboolean dsel_flag)
@@ -1617,8 +1853,7 @@ void create_filesel(gchar *optarg, gboolean dsel_flag)
 
 	font_init();
 
-	if (dialog_compat)
-		gtk_rc_parse_string(FIXED_FONT_RC_STRING);
+	parse_rc_file();
 
 	/* Create a file selector and update Xdialog structure accordingly */
 	Xdialog.window = gtk_file_selection_new(Xdialog.title);
@@ -1694,6 +1929,8 @@ void create_filesel(gchar *optarg, gboolean dsel_flag)
 
 	/* Default exit code */
 	Xdialog.exit_code = 255;
+
+	set_timeout();
 }
 
 
@@ -1707,13 +1944,16 @@ void create_colorsel(gchar *optarg)
 
 	font_init();
 
-	if (dialog_compat)
-		gtk_rc_parse_string(FIXED_FONT_RC_STRING);
+	parse_rc_file();
 
 	/* Create a color selector and update Xdialog structure accordingly */
 	Xdialog.window = gtk_color_selection_dialog_new(Xdialog.title);
 	colorsel = GTK_COLOR_SELECTION_DIALOG(Xdialog.window);
+#ifdef USE_GTK2
+	Xdialog.vbox = GTK_BOX(gtk_widget_get_ancestor(colorsel->colorsel, gtk_box_get_type()));
+#else
 	Xdialog.vbox = GTK_BOX(colorsel->main_vbox);
+#endif
 
 	/* Set the text */
 	box = gtk_widget_get_ancestor(set_label(optarg, TRUE), gtk_box_get_type());
@@ -1777,6 +2017,8 @@ void create_colorsel(gchar *optarg)
 
 	/* Default exit code */
 	Xdialog.exit_code = 255;
+
+	set_timeout();
 }
 
 
@@ -1789,8 +2031,7 @@ void create_fontsel(gchar *optarg)
 
 	font_init();
 
-	if (dialog_compat)
-		gtk_rc_parse_string(FIXED_FONT_RC_STRING);
+	parse_rc_file();
 
 	/* Create a font selector and update Xdialog structure accordingly */
 	Xdialog.window = gtk_font_selection_dialog_new(Xdialog.title);
@@ -1860,6 +2101,8 @@ void create_fontsel(gchar *optarg)
 
 	/* Default exit code */
 	Xdialog.exit_code = 255;
+
+	set_timeout();
 }
 
 
@@ -1901,6 +2144,8 @@ void create_calendar(gchar *optarg, gint day, gint month, gint year)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, calendar_timeout, NULL);
+
+	set_timeout();
 }
 
 
@@ -1934,6 +2179,8 @@ void create_timebox(gchar *optarg, gint hours, gint minutes, gint seconds)
 
 	if (Xdialog.interval > 0)
 		Xdialog.timer = gtk_timeout_add(Xdialog.interval, timebox_timeout, NULL);
+
+	set_timeout();
 }
 
 
